#ifndef ZONE_STRUCT_H
#define ZONE_STRUCT_H

/* ------------------------- BASIC STRUCTURES ------------------------- */

typedef struct ZONE_POINT { 
  double x;
  double y;
} ZONE_POINT;

typedef struct ZONE_CONFIG {
  double x;
  double y;
  double theta;
} ZONE_CONFIG;

typedef enum ZONE_BOOL {
  ZONE_TRUE=1,
  ZONE_FALSE=0
} ZONE_BOOL;

typedef struct ZONE_INIT {
  float robotDiameter;  /* Value to enlarge or reduce obstacles 
			   Warning : same value used for all mobiles !! */
  ZONE_BOOL simulFlag;  /* Only for the mobile ZONE_MOBILE_ROBOT */
  GEN_STRING64 posPosterName; /* Only for the mobile ZONE_MOBILE_ROBOT */
  GEN_STRING128 zonePathFiles; /* can also be done with request ReadZoneFile */
} ZONE_INIT;


/* ------------------------- ZONE DESCRIPTION ------------------------- */

typedef enum ZONE_CLASS {
/* ZONE_TOPO : environnement partition in zones.
   The robot is in the zone iff the center of the robot belongs 
   to the given polygon. */
  ZONE_TOPO,
/* ZONE_STRESS : particular zones in "ZONE_TOPO".
   The robot is in the zone if one part of the robot touches
   the polygon. 
   To optimize this test, the module enlarges the given 
   polygon according to robot radius and then can just tests
   if the center of the robot belongs to the enlarged polygon.
   WARNING : OK for circular robots only
   In case of non-holonomic rectangular robot, the width of the robot 
   instead of the radius is often a good approximation.
*/  ZONE_STRESS
} ZONE_CLASS;
#define ZONE_NB_CLASS 2

#define ZONE_CLASS_NAMES {"ZONE_TOPO", "ZONE_STRESS"}

typedef enum ZONE_MOBILE_ID {
  ZONE_MOBILE_ROBOT,
  ZONE_MOBILE_1,
  ZONE_MOBILE_2,
  ZONE_MOBILE_3
} ZONE_MOBILE_ID;
#define ZONE_NBMAX_MOBILES 4

typedef enum ZONE_TYPE {
  ZONE_TARGET,
  ZONE_SPECIAL,
  ZONE_VIRTUAL_OBST
} ZONE_TYPE;
#define ZONE_NB_TYPES 3

#define ZONE_TYPE_NAMES {"ZONE_TARGET", "ZONE_SPECIAL", "ZONE_VIRTUAL_OBST"}
#define ZONE_FILE_NAMES {"targets.zone", "specials.zone", "virtual_obsts.zone"}

typedef struct ZONE_FILE {
  ZONE_TYPE type;
  int  unused;
  GEN_STRING128 fileName;
} ZONE_FILE;

typedef enum ZONE_STATE {
  ZONE_INSIDE_POLY,
  ZONE_OUTSIDE_POLY
} ZONE_STATE;

#define ZONE_NBMAX_ZONES     40 /* nb max of polygonal zone of a given category */
#define ZONE_NBMAX_PTS      500 /* nb max of pts for all zones of a given category */

typedef struct ZONE_DESC {
  int id;
  ZONE_CLASS class;
  GEN_STRING32 name;
  ZONE_CONFIG finalPos; /* Final position of the robot */
  ZONE_CONFIG target; /* Target position in the zone, for robot orientation */
} ZONE_DESC;

typedef struct ZONE_IDENTITY {
  ZONE_MOBILE_ID mobileId;
  int unused;
  int id;
  ZONE_TYPE type;
  GEN_STRING32 name;
} ZONE_IDENTITY;

typedef struct ZONES_OCCUPIED {
  int nbZones;
  int nbOccupiedZones;
  ZONE_STATE occupiedFlag[ZONE_NBMAX_ZONES]; /* a flag for each zone */
  GEN_STRING32 occupiedList[ZONE_NBMAX_ZONES]; /* list of the nbOccupiedZones occupied zone */
} ZONES_OCCUPIED;

typedef struct ZONES_MAP {
  int nbZones;
  int nbTotalPoints;
  ZONE_DESC desc[ZONE_NBMAX_ZONES];
  int nbPtsPerZone[ZONE_NBMAX_ZONES];
  ZONE_POINT points[ZONE_NBMAX_PTS];
} ZONES_MAP;

typedef struct ZONE_PARAMS {
  ZONE_STATE occupiedFlag;
  int nbPts;
  int firstPt; /* firstPt of the first point of the zone, in the poster ZonesMap */
  int lastPt;
  ZONE_DESC zoneDesc;
} ZONE_PARAMS;

typedef struct ZONE_ROBOT_TO_TARGET {
  ZONE_MOBILE_ID mobileId;
  int unused;
  double dist;  
  double thetaRel;
  double thetaAbs;
} ZONE_ROBOT_TO_TARGET;

typedef enum ZONE_MAP_SOURCE {
  ZONE_FROM_POSTER,
  ZONE_FROM_FILE
} ZONE_MAP_SOURCE;

typedef struct ZONE_SEGLOC_MAP {
  ZONE_MAP_SOURCE source;
  int unused;
  GEN_STRING32 name;
} ZONE_SEGLOC_MAP;


/* ---------------------------- ZONE MONITORING ----------------------------- */


typedef struct ZONE_MOBILE_DESC {
  ZONE_BOOL monitorZoneType[ZONE_NB_TYPES];
  int unused;                     /* because ZONE_NB_TYPE = 3 */
  GEN_STRING64 posPosterName;
} ZONE_MOBILE_DESC;
  
typedef struct ZONE_MOBILE_AND_ZONE_ID {
  ZONE_MOBILE_ID mobileId;
  int zoneId;
} ZONE_MOBILE_AND_ZONE_ID;

typedef struct ZONE_MOBILE_AND_ZONE_TYPE {
  ZONE_MOBILE_ID mobileId;
  ZONE_TYPE zoneType;
} ZONE_MOBILE_AND_ZONE_TYPE;

typedef struct ZONE_TEST_INPUT {
  ZONE_MOBILE_ID mobileId;
  int unused;
  int zoneId;
  ZONE_TYPE zoneType;
  GEN_STRING32 zoneName;
} ZONE_TEST_INPUT;

typedef struct ZONE_TEST_OUTPUT {
  ZONE_MOBILE_ID mobileId;
  ZONE_BOOL zoneTest;
} ZONE_TEST_OUTPUT;



/* -- OBST IN NAV MAP DETECTION ---------------------------------------------- */

typedef struct ZONE_OBSTS_FIELD {
  double range;       /* visibility distance in meters from robot center */
  double direction;   /* direction of the mean angle of the angular sector 
			 in the robot frame */
  double spreadAngle; /* angular sector within obsts are monitor */
} ZONE_OBSTS_FIELD;

typedef enum ZONE_OBST_TYPE {
  ZONE_STATIC_OBST,  /* obst from the map */
  ZONE_DYN_OBST,     /* obst not in the map */
  ZONE_FREE_WAY      /* ie, way with a width > robot diameter */
} ZONE_OBST_TYPE;

typedef struct ZONE_OBST {
  ZONE_OBST_TYPE type; 
  int unused;
  double distMin;
  double angleMean;  /* in robot frame */
  double width;      /*  */
} ZONE_OBST;

#define ZONE_NB_MAX_OBSTS 20
#define ZONE_NB_MAX_DYN_OBSTS 10

typedef enum ZONE_OBST_STATE {
  ZONE_BLOCKED,
  ZONE_HIGH_CLUTTERED,
  ZONE_LOW_CLUTTERED,
  ZONE_FREE
} ZONE_OBST_STATE;

typedef struct ZONE_OBSTS_IN_NAV_MAP_HEADER {
  ZONE_OBST_STATE obstState;
  int id;          /* "date" or fresh flag */
  int nbObsts;     /* number of obstacles (map + dyn) */
  int nbDynObsts;  /* number of non-map obstacles (ie, dyn) */
  int nbFreeWays;  /* number of free ways large enough for the robot */
  int unused;
  double obstsLengthSumCart;
  double freeWaysLengthSumCart;
  double theoricalFreeWaysLengthSumCart;
  double obstsLengthSumPolar;
  double freeWaysLengthSumPolar;
  double theoricalFreeWaysLengthSumPolar;
  int clutteredPercentCart;
  int clutteredPercentPolar;
  ZONE_OBSTS_FIELD obstsField;
} ZONE_OBSTS_IN_NAV_MAP_HEADER;

typedef struct ZONE_OBSTS_IN_NAV_MAP_OBSTS {
  ZONE_OBST obst[ZONE_NB_MAX_OBSTS]; /* dyn obsts + map obsts + free ways */
} ZONE_OBSTS_IN_NAV_MAP_OBSTS;

typedef struct ZONE_OBSTS_IN_NAV_MAP {
  ZONE_OBSTS_IN_NAV_MAP_HEADER header;
  ZONE_OBSTS_IN_NAV_MAP_OBSTS obsts;
/*   SICK_SEG  segs[ZONE_NB_MAX_DYN_OBSTS]; */
} ZONE_OBSTS_IN_NAV_MAP;


typedef struct ZONE_GET_OUT_FROM_OBST {
  double minDist;
  ZONE_POINT pos;
} ZONE_GET_OUT_FROM_OBST;

#endif /* ZONE_STRUCT_H */

